/* global Bdf,$$ */
/**
 * Objet global définissant l'espace de nom Pioche
 * 
 * @namespace Pioche
 */
var Pioche = {};

Pioche.ARGS = {
  appelant: "",  
  limit: -1,
  count: -1,
  separator: "",
  wanted: "",
  croisement: "",
  json: "",
  subsets: "",
  param: ""
};

Pioche.JSONITEM_MAP = new Map();

Pioche.init = function (args) {
    var piocheId = Bdf.generateId();
    var jsonData = null;
    $$(args.clientId).html(Bdf.render("pioche:client", {
        args: args,
        piocheId: piocheId
    }));
    _$$("confirm").click(_updateAppelant);
    _$$("cancel").click(function () {
        window.close();
    });
    _$$("select", "available").click(function () {
        _selection(this);
    }).keydown(function (event) {
         if (event.key === "Enter") {
            _selection(this);
        }
    });
    _$$("select", "selected").click(function () {
        _deselection(this);
    }).keydown(function (event) {
         if (event.key === "Enter") {
            _deselection(this);
        }
    });
    if (args.limit === 1) {
        _$$("select", "selected").attr("size", "2");
    }
    if ((args.count > -1) && (args.count < 31)) {
        _$$("header").addClass("hidden");
        _$$("form").ajaxSubmit({
            dataType:  'json',
            success:   _processJson
        });
    } else {
        _$$("form").ajaxForm({
            dataType:  'json',
            success:   function (data, textStatus, jqXHR, $form) {
                _processJson(data);
            }
        });
        _$$("qinput").focus();
    }
    
    
    function _$$(id) {
        if (arguments.length > 1) {
            id = Array.from(arguments).join("_");
        }
        return $$(piocheId, id);
    }
    
    function _updateAppelant() {
        this.disabled = true;
        let callback = window.opener.Bdf.getAppelantCallback(args.appelant);
        if (callback) {
            let selectionArray = new Array();
            _$$("select", "selected").find("option").each(function (index, option) {
                let jsonItem = Pioche.JSONITEM_MAP.get(option.value);
                 if (jsonItem) {
                    selectionArray.push(jsonItem);
                }
            });
            callback(selectionArray);
        } else {
            _defaultAppelantUpdate();
        }
        window.close();
    }
    
    function _defaultAppelantUpdate() {
        let limit = args.limit;
        let newValue = "";
        _$$("select", "selected").find("option").each(function (index, option) {
            newValue = newValue + option.value;
            if (limit !== 1) newValue = newValue + "; ";
        });               
        let element = window.opener.document.getElementById(args.appelant);
        if (limit !== 1) {
            let currentValue = element.value;
            if (_testSemicolon(currentValue)) {
                currentValue = currentValue + "; ";
            }
            newValue = currentValue + newValue;
        }
        element.value = newValue;
    }

    function _testSemicolon(currentText) {
        let size = currentText.length;
        if (size === 0) return false;
        for (let i = size-1;i >= 0;i --) {
            let carac = currentText.charAt(i);
            if (carac !== " ") {
                if (carac === ";") return false;
                else return true;
            }
        }
        return false;
    }

    function _processJson(data) {
        jsonData = data;
        for(let jsonItem of jsonData.array) {
            let keyAndText = _getKeyAndText(jsonItem);
            Pioche.JSONITEM_MAP.set(keyAndText.key, jsonItem);
        }
        let length = data.length;
        _$$("count", "available").text(length);
        let $tabs = _$$("tabs", "available").empty();
        if (length === 0) {
            alert(Bdf.Loc.get("_ warning.pioche.emptyresult"));
        } else {
            let seuilvisibilite = Math.ceil(length/10);
            if (seuilvisibilite > 1) {
                for (let k = 0; k < seuilvisibilite;k++) {
                   $tabs.append(Bdf.render("pioche:tab", {
                       piocheId: piocheId,
                       number: k,
                       disabled: (k === 0)
                   }));
                   _$$("tab", k).click(function () {
                       _activeTab(k, $(this));
                   });
                }
                _activeTab(0, _$$("tab", 0));
            } else {
                _updateAvailable(0);
            }
        }
    }

    function _activeTab(dizaine, $tab) {
        Bdf.$({role: "tab"}).prop("disabled", false);
        $tab.prop("disabled", true);
        _updateAvailable(dizaine);
    }

    function _updateAvailable(dizaine) {
        let $avalaibleSelect = _$$("select", "available").empty();
        let plafond = Math.min(jsonData.length, 10*(dizaine + 1));
        let plancher = 10*dizaine;
        for (let k = plancher; k < plafond;k ++) {
            let keyAndText = _getKeyAndText(jsonData.array[k]);
            $avalaibleSelect.append(Bdf.render("pioche:option", {
                value: keyAndText.key,
                text: keyAndText.text
            }));
        }
    }

    function _selection(select) {
        let idx = select.selectedIndex;
        if (idx === -1) return;
        let option = select.options[idx];
        let value = option.value;
        let $selectedSelect = _$$("select", "selected");
        for (let opt of $selectedSelect[0].options) {
            if (opt.value === value) return;
        }
        $selectedSelect.append(Bdf.render("pioche:option", {
            value: value,
            text: option.text
        }));
        _checkSelect();
    }

    function _checkSelect() {
        let count = _$$("select", "selected").find("option").length;
        let limit = args.limit;
        if((limit !== -1) && (count >= limit)) {
            _$$("select", "available").prop("disabled", true);
        }
        else {
            _$$("select", "available").prop("disabled", false);
        }
        _$$("confirm").prop("disabled", (count === 0));
        _$$("count", "selected").text(count);
    }

    function _deselection(select) {
        let idx = select.selectedIndex;
        if (idx === -1) return;
        select.remove(idx);
        select.selectedIndex = -1;
        _checkSelect();
    }
    
    function _getKeyAndText(jsonItem) {
        let key = "";
        let text = "";
        switch(args.wanted) {
            case "id":
                key = jsonItem.id;
                if (jsonItem.code) {
                    text = jsonItem.code;
                } else {
                    text = "" + key;
                }
                break;
            case "code-id":
                if (jsonItem.code) {
                    key = jsonItem.code;
                } else {
                    key = jsonItem.id;
                }
                text = "" + key;
                break;
            case "code-title":
                 if (jsonItem.code) {
                    key = jsonItem.code;
                    text = "" + key;
                } else if (jsonItem.title) {
                    key = jsonItem.title;
                    text = "";
                } else {
                    key = jsonItem.id;
                    text = "" + key;
                }
                break;
        }
        if (jsonItem.title) {
            if (text) {
                text = text + args.separator + jsonItem.title;
            } else {
                text = jsonItem.title;
            }
        }
        if (jsonItem.description) {
            text = text + " (" + jsonItem.description + ")";
        }
        return {
            key: key.toString(),
            text: text
        };
    }
};

$(function () {
    Bdf.initTemplates();
    Pioche.init(Pioche.ARGS);
});
